/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/


#define MAX_CHART_ENTRY 50

#include "CollectionView/collectionscene.h"
#include "CollectionModel/collectionmodel.h"

#include "CollectionView/button_item.h"

#include "debug.h"


/*
********************************************************************************
*                                                                              *
*    Class CollectionScene                                                     *
*        -> dashboard method                                                   *
********************************************************************************
*/
void CollectionScene::populateDashBoardScene()
{
    Debug::debug() << " ---- CollectionScene::populateDashBoardScene";
    
    QGraphicsView* view = qobject_cast<QGraphicsView*> (_parentWidget);
    int utilWidth = view->verticalScrollBar()->isVisible() ? (view->width() - view->verticalScrollBar()->width())  : view->width();

    
    //! header item
    HeaderItem* header = new HeaderItem();
    header->_width = utilWidth - 10;
    header->setText( tr("Dashboard") );
    header->setPos(0, 5);
    addItem(header);
    
    //! menu button    
    QMenu* menu = new QMenu();
    menu->addAction(m_actions.value("cover"));

    ButtonMenuItem* ui_menu_button = new ButtonMenuItem();
    ui_menu_button->setPixmap(QPixmap(":/images/settings-white-48x48.png"));
    ui_menu_button->setToolTip(tr("view actions"));
    ui_menu_button->setMenu(menu);
    ui_menu_button->setPos(utilWidth-10-ui_menu_button->width(), 5);
    addItem(ui_menu_button);
    
    //! content    
    populateMostPlayedAlbum(itemsBoundingRect().height() + 40);
    populateTopRatedAlbum(itemsBoundingRect().height() + 40);
    populateMostPlayedArtist(itemsBoundingRect().height() + 40);
    populateTopRatedArtist(itemsBoundingRect().height() + 40);
}



/*******************************************************************************
    album part
*******************************************************************************/
void CollectionScene::populateMostPlayedAlbum(int YPos)
{
    int albumRow  = 0;
    int Column    = 0;
    int nbItem    = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    int maxPlaycount = 0;

    int itemCount = 0;

    CategorieGraphicItem *category = new CategorieGraphicItem();
    category->_catName = QString(tr("Most played albums"));
    category->_width   = _parentWidget->width() -20;
    category->setPos(0 ,YPos);
    addItem(category);

    //! get Album List from CollectionModel
    QList<MEDIA::AlbumPtr> albums = _collectionModel->albumItemList;

    //! sort
    qSort(albums.begin(), albums.end(), MEDIA::compareAlbumItemPlaycount);

    //! loop over album MediaItem
    foreach(MEDIA::AlbumPtr album, albums)
    {
      if(album->playcount == 0 || itemCount == MAX_CHART_ENTRY) break;

      if(maxPlaycount == 0)
        maxPlaycount = album->playcount;

      if( !_collectionModel->isAlbumFiltered(album) ) continue;

      AlbumGraphicItem_v3 *album_item = new AlbumGraphicItem_v3();
      album_item->media = album;
      album_item->setPos(4+160*Column, YPos + 35 + albumRow*190);

      addItem(album_item);
      itemCount++;

      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        albumRow++;
      }
    }

    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , YPos + 50);
      addItem(info);
    }
}

void CollectionScene::populateTopRatedAlbum(int YPos)
{
    int albumRow  = 0;
    int Column    = 0;
    int nbItem    = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    int itemCount = 0;


    CategorieGraphicItem *category = new CategorieGraphicItem();
    category->_catName = QString(tr("Top rated albums"));
    category->_width   = _parentWidget->width()-20;
    category->setPos(0 ,YPos);
    addItem(category);

    //! get Album List from CollectionModel
    QList<MEDIA::AlbumPtr> albums = _collectionModel->albumItemList;

    //! sort
    qSort(albums.begin(), albums.end(), MEDIA::compareAlbumItemRating);

    //! loop over album MediaItem
    foreach(MEDIA::AlbumPtr album, albums)
    {
      if(album->rating == 0.0 || itemCount == MAX_CHART_ENTRY) break;

      if( !_collectionModel->isAlbumFiltered(album) ) continue;

      AlbumGraphicItem_v4 *album_item = new AlbumGraphicItem_v4();
      album_item->media = album;
      album_item->setPos(4+160*Column, YPos + 35 + albumRow*190);

      addItem(album_item);
      itemCount++;

      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        albumRow++;
      }
    }

    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , YPos + 50);
      addItem(info);
    }
}



/*******************************************************************************
    artist part
*******************************************************************************/
void CollectionScene::populateMostPlayedArtist(int YPos)
{
    int artistRow  = 0;
    int Column     = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    int itemCount    = 0;

    CategorieGraphicItem *category = new CategorieGraphicItem();
    category->_catName = QString(tr("Most played artists"));
    category->_width   = _parentWidget->width()-20;
    category->setPos(0 ,YPos);
    addItem(category);

    //! get artists List from CollectionModel
    QList<MEDIA::ArtistPtr> artists;
    for (int i=0 ; i < _collectionModel->rootItem()->childCount(); i++ ) {
       artists << MEDIA::ArtistPtr::staticCast(_collectionModel->rootItem()->child(i));
     }

    //! sort
    qSort(artists.begin(), artists.end(), MEDIA::compareArtistItemPlaycount);


    //! loop over artist MediaItem
    foreach(MEDIA::ArtistPtr artist, artists)
    {
      if(artist->playcount == 0 || itemCount == MAX_CHART_ENTRY) break;

      if( !_collectionModel->isArtistFiltered(artist) ) continue;


      ArtistGraphicItem_v2 *artist_item = new ArtistGraphicItem_v2();
      artist_item->media = artist;
      artist_item->setPos(4+160*Column, YPos + 35 + artistRow*190);

      addItem(artist_item);
      itemCount++;

      //! ALBUM COVER LOOP
      artist->album_covers.clear();
      for(int j = artist->childCount()-1 ; j >= 0; j--) {
          MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(artist->child(j));
          if(!_collectionModel->isAlbumFiltered(album) ) continue;

          artist->album_covers.prepend(album);

          //!WARNING limite de l'affichage  6 cover max
          if(artist->album_covers.size() >=6) break;
      }

      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        artistRow++;
      }
    }

    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , YPos + 50);
      addItem(info);
    }
}


void CollectionScene::populateTopRatedArtist(int YPos)
{
    int artistRow  = 0;
    int Column     = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    int itemCount = 0;


    CategorieGraphicItem *category = new CategorieGraphicItem();
    category->_catName = QString(tr("Top rated artists"));
    category->_width   = _parentWidget->width()-20;
    category->setPos(0 ,YPos);
    addItem(category);

    //! get artists List from CollectionModel
    QList<MEDIA::ArtistPtr> artists;
    for (int i=0 ; i < _collectionModel->rootItem()->childCount(); i++ ) {
       artists << MEDIA::ArtistPtr::staticCast(_collectionModel->rootItem()->child(i));
     }

    //! sort
    qSort(artists.begin(), artists.end(), MEDIA::compareArtistItemRating);

    //! loop over artist MediaItem
    foreach(MEDIA::ArtistPtr artist, artists)
    {
      if(artist->rating == 0.0 || itemCount == MAX_CHART_ENTRY) break;

      if( !_collectionModel->isArtistFiltered(artist) ) continue;


      ArtistGraphicItem_v3 *artist_item = new ArtistGraphicItem_v3();
      artist_item->media = artist;
      artist_item->setPos(4+160*Column, YPos + 35 + artistRow*190);

      addItem(artist_item);
      itemCount++;

      //! ALBUM COVER LOOP
      artist->album_covers.clear();
      for(int j = artist->childCount()-1 ; j >= 0; j--) {
          MEDIA::AlbumPtr album = MEDIA::AlbumPtr::staticCast(artist->child(j));
          if(!_collectionModel->isAlbumFiltered(album) ) continue;

          artist->album_covers.prepend(album);

          //!WARNING limite de l'affichage  6 cover max
          if(artist->album_covers.size() >=6) break;
      }


      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        artistRow++;
      }
    }

    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *info = new InfoGraphicItem();
      info->_text = tr("No entry found");
      info->_width   = _parentWidget->width();
      info->setPos( 0 , YPos + 50);
      addItem(info);
    }
}

